@@ -18,7 +18,7 @@ where
 -- representations rather than the terms themselves, leads to
 -- identical results.
 
-#if defined (GHCLIB_API_811)
+#if defined (GHCLIB_API_HEAD) || defined(GHCLIB_API_902) || defined (GHCLIB_API_900)
 import GHC.Utils.Outputable
 #else
 import Outputable
@@ -39,7 +39,11 @@ extendInstances = HsExtendInstances
 -- string representations.
 toStr :: Data a => HsExtendInstances a -> String
 toStr (HsExtendInstances e) =
+#if defined(GHCLIB_API_HEAD) || defined(GHCLIB_API_902)
+  showPprUnsafe $ showAstData BlankSrcSpan BlankEpAnnotations e
+#else
   showSDocUnsafe $ showAstData BlankSrcSpan e
+#endif
 
 instance Data a => Eq (HsExtendInstances a) where (==) a b = toStr a == toStr b
 instance Data a => Ord (HsExtendInstances a) where compare = compare `on` toStr
@@ -52,4 +56,9 @@ astListEq as bs = length as == length bs && all (uncurry astEq) (zip as bs)
 
 -- Use 'ppr' for 'Show'.
 instance Outputable a => Show (HsExtendInstances a) where
-  show (HsExtendInstances e) =  showSDocUnsafe $ ppr e
+  show (HsExtendInstances e) =
+#if defined(GHCLIB_API_HEAD) || defined(GHCLIB_API_902)
+    showPprUnsafe $ ppr e
+#else
+    showSDocUnsafe $ ppr e
+#endif
