@@ -14,7 +14,7 @@ module Language.Haskell.GhclibParserEx.GHC.Hs.ImpExp(
   )
 where
 
-#if defined (GHCLIB_API_811)
+#if defined (GHCLIB_API_HEAD) || defined(GHCLIB_API_902) || defined (GHCLIB_API_900)
 import GHC.Hs.ImpExp
 import GHC.Types.Name.Reader
 #elif defined (GHCLIB_API_810)
@@ -29,7 +29,7 @@ isPatSynIE :: IEWrappedName RdrName -> Bool
 isPatSynIE IEPattern{} = True
 isPatSynIE _ = False
 
-#if defined(MIN_VERSION_ghc_lib_parser)
+#if defined (MIN_VERSION_ghc_lib_parser)
 #  if !MIN_VERSION_ghc_lib_parser(1,  0,  0) || MIN_VERSION_ghc_lib_parser(8, 10, 0)
 isImportQualifiedPost :: ImportDeclQualifiedStyle -> Bool
 isImportQualifiedPost QualifiedPost = True
