@@ -8,11 +8,16 @@ module Language.Haskell.GhclibParserEx.GHC.Utils.Outputable (
 )
 where
 
-#if defined (GHCLIB_API_811)
+#if defined (GHCLIB_API_HEAD) || defined(GHCLIB_API_902) || defined (GHCLIB_API_900)
 import GHC.Utils.Outputable
 #else
 import Outputable
 #endif
 
 unsafePrettyPrint :: Outputable a => a -> String
-unsafePrettyPrint = showSDocUnsafe . ppr
+unsafePrettyPrint =
+#if defined (GHCLIB_API_HEAD)
+  showPprUnsafe . ppr
+#else
+  showSDocUnsafe . ppr
+#endif
