@@ -16,6 +16,12 @@ import System.IO.Extra
 import Control.Monad
 import Data.List.Extra
 import Data.Maybe
+import Data.Generics.Uniplate.Data
+#if defined (GHCLIB_API_HEAD)
+import GHC.Data.Bag
+import GHC.Driver.Errors.Types
+import GHC.Types.Error
+#endif
 
 import Language.Haskell.GhclibParserEx.Dump
 import Language.Haskell.GhclibParserEx.Fixity
@@ -25,6 +31,7 @@ import Language.Haskell.GhclibParserEx.GHC.Hs
 import Language.Haskell.GhclibParserEx.GHC.Hs.ExtendInstances
 import Language.Haskell.GhclibParserEx.GHC.Hs.Expr
 import Language.Haskell.GhclibParserEx.GHC.Hs.Pat
+import Language.Haskell.GhclibParserEx.GHC.Hs.Type
 -- We only test 'isImportQualifiedPost' at this time which requires >=
 -- 8.10; avoid unused import warning.
 #if defined (MIN_VERSION_ghc_lib_parser)
@@ -38,16 +45,24 @@ import Language.Haskell.GhclibParserEx.GHC.Driver.Flags()
 import Language.Haskell.GhclibParserEx.GHC.Driver.Session
 import Language.Haskell.GhclibParserEx.GHC.Types.Name.Reader
 
-#if defined (GHCLIB_API_811) || defined (GHCLIB_API_810)
+#if defined (GHCLIB_API_HEAD) || defined (GHCLIB_API_902) || defined (GHCLIB_API_900) || defined (GHCLIB_API_810)
 import GHC.Hs
 #else
 import HsSyn
 #endif
-#if defined (GHCLIB_API_811)
+#if defined (GHCLIB_API_HEAD) || defined (GHCLIB_API_902) || defined (GHCLIB_API_900)
 import GHC.Types.SrcLoc
 import GHC.Driver.Session
 import GHC.Parser.Lexer
+# if !defined (GHCLIB_API_HEAD)
 import GHC.Utils.Outputable
+#endif
+#  if !defined (GHCLIB_API_900)
+import GHC.Driver.Ppr
+#    if !defined (GHCLIB_API_HEAD)
+import GHC.Parser.Errors.Ppr
+#    endif
+#  endif
 import GHC.Utils.Error
 import GHC.Types.Name.Reader
 import GHC.Types.Name.Occurrence
@@ -77,6 +92,7 @@ tests = testGroup " All tests"
   , fixityTests
   , extendInstancesTests
   , expressionPredicateTests
+  , typePredicateTests
   , patternPredicateTests
   , dynFlagsTests
   , nameTests
@@ -88,13 +104,37 @@ makeFile relPath contents = do
     writeFile relPath contents
     return relPath
 
+#if defined (GHCLIB_API_HEAD)
+chkParseResult :: (DynFlags -> Bag (MsgEnvelope GhcMessage)  -> String) -> DynFlags -> ParseResult a -> IO ()
+#else
 chkParseResult :: (DynFlags -> WarningMessages -> String) -> DynFlags -> ParseResult a -> IO ()
+#endif
 chkParseResult report flags = \case
     POk s _ -> do
+#if defined (GHCLIB_API_HEAD)
+      let (wrns, errs) = getPsMessages s
+#elif defined (GHCLIB_API_902)
+      let (wrns, errs) = getMessages s
+#else
       let (wrns, errs) = getMessages s flags
+#endif
       when (not (null errs) || not (null wrns)) $
+#if defined (GHCLIB_API_HEAD)
+        assertFailure (
+          report flags (getMessages (GhcPsMessage <$> wrns)) ++
+          report flags (getMessages (GhcPsMessage <$> errs))
+        )
+
+#elif defined (GHCLIB_API_902)
+        assertFailure (report flags (fmap pprWarning wrns) ++ report flags (fmap pprError errs))
+#else
         assertFailure (report flags wrns ++ report flags errs)
-#if defined (GHCLIB_API_811) || defined (GHCLIB_API_810)
+#endif
+#if defined (GHCLIB_API_HEAD)
+    PFailed s -> assertFailure (report flags $ getMessages (GhcPsMessage <$> snd (getPsMessages s)))
+#elif defined (GHCLIB_API_902)
+    PFailed s -> assertFailure (report flags $ fmap pprError (snd (getMessages s)))
+#elif defined (GHCLIB_API_900) || defined (GHCLIB_API_810)
     PFailed s -> assertFailure (report flags $ snd (getMessages s flags))
 #else
     PFailed _ loc err -> assertFailure (report flags $ unitBag $ mkPlainErrMsg flags loc err)
@@ -102,44 +142,53 @@ chkParseResult report flags = \case
 
 parseTests :: TestTree
 parseTests = testGroup "Parse tests"
-  [ testCase "Module" $
-      chkParseResult report flags $
+  [
+    testCase "Module" $
+      chkParseResult report flags (
         parseModule (unlines
           [ "module Foo (readMany) where"
           , "import Data.List"
           , "import Data.Maybe"
           , "readMany = unfoldr $ listToMaybe . concatMap reads . tails"
-          ]) flags
+          ]) flags)
+  , testCase "Module" $
+      chkParseResult report flags (
+        parseModule (unlines
+          [ "module Foo (readMany) where"
+          , "import Data.List"
+          , "import Data.Maybe"
+          , "readMany = unfoldr $ listToMaybe . concatMap reads . tails"
+          ]) flags)
   , testCase "Signature" $
-      chkParseResult report flags $
+      chkParseResult report flags (
         parseSignature (unlines
           [ "signature Str where"
           , "data Str"
           , "empty :: Str"
           , "append :: Str -> Str -> Str"
-          ]) flags
+          ]) flags)
   , testCase "Import" $
-      chkParseResult report flags $
-        parseImport "import qualified \"foo-lib\" Foo as Bar hiding ((<.>))" flags
+      chkParseResult report flags (
+        parseImport "import qualified \"foo-lib\" Foo as Bar hiding ((<.>))" flags)
   , testCase "Statement" $
-      chkParseResult report flags $
-        parseStatement "Foo foo <- bar" flags
+      chkParseResult report flags (
+        parseStatement "Foo foo <- bar" flags)
   , testCase "Backpack" $
-      chkParseResult report flags $
+      chkParseResult report flags (
         parseBackpack (unlines
           [ "unit main where"
           , "  module Main where"
           , "    main = putStrLn \"Hello world!\""
-          ]) flags
+          ]) flags)
   , testCase "Expression" $
-      chkParseResult report flags $
-        parseExpression "unfoldr $ listToMaybe . concatMap reads . tails" flags
+      chkParseResult report flags (
+        parseExpression "unfoldr $ listToMaybe . concatMap reads . tails" flags)
   , testCase "Declaration (1)" $
-      chkParseResult report flags $
-        parseDeclaration "fact n = if n <= 1 then 1 else n * fact (n - 1)" flags
+      chkParseResult report flags (
+        parseDeclaration "fact n = if n <= 1 then 1 else n * fact (n - 1)" flags)
   , testCase "Declaration (2)" $ -- Example from https://github.com/ndmitchell/hlint/issues/842.
-      chkParseResult report flags $
-        parseDeclaration "infixr 4 <%@~" flags
+      chkParseResult report flags (
+        parseDeclaration "infixr 4 <%@~" flags)
   , testCase "File" $ withTempDir $ \tmpDir -> do
       foo <- makeFile (tmpDir </> "Foo.hs") $ unlines
         ["{-# LANGUAGE ScopedTypeVariables #-}"
@@ -151,13 +200,17 @@ parseTests = testGroup "Parse tests"
       s <- readFile' foo
       parsePragmasIntoDynFlags flags ([], []) foo s >>= \case
         Left msg -> assertFailure msg
-        Right flags -> chkParseResult report flags $ parseFile foo flags s
+        Right flags -> chkParseResult report flags (parseFile foo flags s)
   ]
   where
-    flags = unsafeGlobalDynFlags
+    flags = defaultDynFlags fakeSettings fakeLlvmConfig
+#if defined (GHCLIB_API_HEAD) || defined (GHCLIB_API_902)
+    report flags msgs = concat [ showSDoc flags msg | msg <- pprMsgEnvelopeBagWithLoc msgs ]
+#else
     report flags msgs = concat [ showSDoc flags msg | msg <- pprErrMsgBagWithLoc msgs ]
+#endif
 
-#if defined(GHCLIB_API_811)
+#if defined (GHCLIB_API_HEAD) || defined (GHCLIB_API_902) || defined (GHCLIB_API_900)
 moduleTest :: String -> DynFlags -> (Located HsModule -> IO ()) -> IO ()
 #else
 moduleTest :: String -> DynFlags -> (Located (HsModule GhcPs) -> IO ()) -> IO ()
@@ -173,6 +226,12 @@ exprTest s flags test =
         POk _ e -> test e
         _ -> assertFailure "parse error"
 
+typeTest :: String -> DynFlags -> (LHsType GhcPs -> IO ()) -> IO ()
+typeTest s flags test =
+      case parseType s flags of
+        POk _ e -> test e
+        _ -> assertFailure "parse error"
+
 patTest :: String -> DynFlags -> (LPat GhcPs -> IO ()) -> IO ()
 patTest s flags test =
       case parsePattern s flags of
@@ -185,15 +244,25 @@ fixityTests = testGroup "Fixity tests"
       exprTest "1 + 2 * 3" flags
         (\e ->
             assertBool "parse tree not affected" $
+#if defined (GHCLIB_API_HEAD) || defined (GHCLIB_API_902)
+              showSDocUnsafe (showAstData BlankSrcSpan BlankEpAnnotations e) /=
+              showSDocUnsafe (showAstData BlankSrcSpan BlankEpAnnotations (applyFixities [] e))
+#else
               showSDocUnsafe (showAstData BlankSrcSpan e) /=
               showSDocUnsafe (showAstData BlankSrcSpan (applyFixities [] e))
+#endif
         )
   , testCase "Pattern" $
       case parseDeclaration "f (1 : 2 :[]) = 1" flags of
         POk _ d ->
           assertBool "parse tree not affected" $
+#if defined (GHCLIB_API_HEAD) || defined (GHCLIB_API_902)
+          showSDocUnsafe (showAstData BlankSrcSpan BlankEpAnnotations d) /=
+          showSDocUnsafe (showAstData BlankSrcSpan BlankEpAnnotations (applyFixities [] d))
+#else
           showSDocUnsafe (showAstData BlankSrcSpan d) /=
           showSDocUnsafe (showAstData BlankSrcSpan (applyFixities [] d))
+#endif
         PFailed{} -> assertFailure "parse error"
   , testCase "fixitiesFromModule" $
       case parseModule "infixl 4 <*!" flags of
@@ -223,6 +292,16 @@ extendInstancesTests = testGroup "Extend instances tests"
   where
     flags = defaultDynFlags fakeSettings fakeLlvmConfig
 
+typePredicateTests :: TestTree
+typePredicateTests = testGroup "Type predicate tests"
+  [ testCase "isKindTyApp" $ test_with_exts [TypeApplications] "K @T" $ assert' . isKindTyApp
+  , testCase "isKindTyApp" $ test_with_exts [TypeApplications] "K T" $ assert' . not . isKindTyApp
+  ]
+  where
+    assert' = assertBool ""
+    test_with_exts exts s = typeTest s (flags exts)
+    flags = foldl' xopt_set (defaultDynFlags fakeSettings fakeLlvmConfig)
+
 expressionPredicateTests :: TestTree
 expressionPredicateTests = testGroup "Expression predicate tests"
   [ testCase "isTag" $ test "foo" $ assert' . isTag "foo"
@@ -231,14 +310,22 @@ expressionPredicateTests = testGroup "Expression predicate tests"
   , testCase "isDot" $ test "f . g" $ \case L _ (OpApp _ _ op _) -> assert' $ isDot op; _ -> assertFailure "unexpected"
   , testCase "isReturn" $ test "return x" $ \case L _ (HsApp _ f _) -> assert' $ isReturn f; _ -> assertFailure "unexpected"
   , testCase "isReturn" $ test "pure x" $ \case L _ (HsApp _ f _) -> assert' $ isReturn f; _ -> assertFailure "unexpected"
+#if defined (GHCLIB_API_HEAD)
+  , testCase "isSection" $ test "(1 +)" $ \case L _ (HsPar _ _ x _) -> assert' $ isSection x; _ -> assertFailure "unexpected"
+#else
   , testCase "isSection" $ test "(1 +)" $ \case L _ (HsPar _ x) -> assert' $ isSection x; _ -> assertFailure "unexpected"
+#endif
+#if defined (GHCLIB_API_HEAD)
+  , testCase "isSection" $ test "(+ 1)" $ \case L _ (HsPar _ _ x _) -> assert' $ isSection x; _ -> assertFailure "unexpected"
+#else
   , testCase "isSection" $ test "(+ 1)" $ \case L _ (HsPar _ x) -> assert' $ isSection x; _ -> assertFailure "unexpected"
+#endif
   , testCase "isRecConstr" $ test "Foo {bar=1}" $ assert' . isRecConstr
   , testCase "isRecUpdate" $ test "foo {bar=1}" $ assert' . isRecUpdate
   , testCase "isVar" $ test "foo" $ assert' . isVar
   , testCase "isVar" $ test "3" $ assert' . not. isVar
   , testCase "isPar" $ test "(foo)" $ assert' . isPar
-  , testCase "isPar" $ test "foo" $ assert' . not. isPar
+  , testCase "isPar" $ test "foo" $ assert' . not . isPar
   , testCase "isApp" $ test "f x" $ assert' . isApp
   , testCase "isApp" $ test "x" $ assert' . not . isApp
   , testCase "isOpApp" $ test "l `op` r" $ assert' . isOpApp
@@ -281,10 +368,17 @@ expressionPredicateTests = testGroup "Expression predicate tests"
   , testCase "isSpliceDecl" $ test "$x" $ assert' . isSpliceDecl . unLoc
   , testCase "isSpliceDecl" $ test "f$x" $ assert' . not . isSpliceDecl . unLoc
   , testCase "isSpliceDecl" $ test "$(a + b)" $ assert' . isSpliceDecl . unLoc
-  , testCase "isQuasiQuote" $ test "[expr|1 + 2|]" $ assert' . isQuasiQuote
-  , testCase "isQuasiQuote" $ test "[expr(1 + 2)]" $ assert' . not . isQuasiQuote
+  , testCase "isQuasiQuoteExpr" $ test "[expr|1 + 2|]" $ assert' . isQuasiQuoteExpr
+  , testCase "isQuasiQuoteExpr" $ test "[expr(1 + 2)]" $ assert' . not . isQuasiQuoteExpr
   , testCase "isWholeFrac" $ test "3.2e1" $ assert' . isWholeFrac . unLoc
   , testCase "isWholeFrac" $ test "3.22e1" $ assert' . not . isWholeFrac . unLoc
+  , testCase "isMDo" $ test_with_exts [ RecursiveDo ] "mdo { pure () }" $ assert' . any isMDo . universeBi
+  , testCase "isListComp (1)" $ test "[ x + y | x <- xs, y <- ys ]" $ assert' . any isListComp . universeBi
+  , testCase "isListComp (2)" $ test_with_exts [ MonadComprehensions ] "[ x + y | x <- xs, y <- ys ]" $ assert' . any isMonadComp . universeBi
+  , testCase "isMonadComp (0)" $ test_with_exts [ MonadComprehensions ] "[ x + y | x <- Just 1, y <- Just 2 ]" $ assert' . not . any isListComp . universeBi
+  , testCase "isMonadComp (1)" $ test_with_exts [ MonadComprehensions ] "[ x + y | x <- Just 1, y <- Just 2 ]" $ assert' . any isMonadComp . universeBi
+  , testCase "isMonadComp (2)" $ test_with_exts [] "[ x + y | x <- Just 1, y <- Just 2 ]" $ assert' . not . any isMonadComp . universeBi
+  , testCase "isMonadComp (3)" $ test_with_exts [] "[ x + y | x <- Just 1, y <- Just 2 ]" $ assert' . any isListComp . universeBi
   , testCase "strToVar" $ assert' . isVar . strToVar $ "foo"
   , testCase "varToStr" $ test "[]" $ assert' . (== "[]") . varToStr
   , testCase "varToStr" $ test "foo" $ assert' . (== "foo") . varToStr
@@ -292,9 +386,17 @@ expressionPredicateTests = testGroup "Expression predicate tests"
   ]
   where
     assert' = assertBool ""
-    test s = exprTest s flags
-    flags = foldl' xopt_set (defaultDynFlags fakeSettings fakeLlvmConfig)
-              [ TemplateHaskell, TemplateHaskellQuotes, QuasiQuotes, TypeApplications, LambdaCase ]
+    test s = exprTest s (flags [])
+    test_with_exts exts s = exprTest s (flags exts)
+    flags exts = foldl' xopt_set (defaultDynFlags fakeSettings fakeLlvmConfig)
+              (exts ++
+                 [ TemplateHaskell
+                 , TemplateHaskellQuotes
+                 , QuasiQuotes
+                 , TypeApplications
+                 , LambdaCase
+                 ]
+              )
 
 patternPredicateTests :: TestTree
 patternPredicateTests = testGroup "Pattern predicate tests"
@@ -306,12 +408,21 @@ patternPredicateTests = testGroup "Pattern predicate tests"
   , testCase "strToPat" $ assert' . (== "[]") . patToStr . strToPat $ "[]"
   , testCase "fromPChar" $ test "'a'" $ assert' . (== Just 'a') . fromPChar
   , testCase "fromPChar" $ test "\"a\"" $ assert' . isNothing . fromPChar
+  , testCase "isSplicePat" $ test "$(varP pylonExPtrVarName)" $ assert' . isSplicePat
   ]
   where
     assert' = assertBool ""
-    test s = patTest s flags
-    flags = foldl' xopt_set (defaultDynFlags fakeSettings fakeLlvmConfig)
-              [ TemplateHaskell, QuasiQuotes, TypeApplications, LambdaCase ]
+    test = test_with_exts []
+    test_with_exts exts s = patTest s (flags exts)
+    flags exts = foldl' xopt_set (defaultDynFlags fakeSettings fakeLlvmConfig)
+              (exts ++
+                 [ TemplateHaskell
+                 , TemplateHaskellQuotes
+                 , QuasiQuotes
+                 , TypeApplications
+                 , LambdaCase
+                 ]
+              )
 
 dynFlagsTests :: TestTree
 dynFlagsTests = testGroup "DynFlags tests"
@@ -336,9 +447,9 @@ dynFlagsTests = testGroup "DynFlags tests"
       -- https://github.com/ndmitchell/hlint/issues/971).
       parsePragmasIntoDynFlags flags ([StarIsType], []) foo s >>= \case
         Left msg -> assertFailure msg
-        Right flags -> chkParseResult report flags $ parseFile foo flags s
+        Right flags -> chkParseResult report flags (parseFile foo flags s)
 #if defined (MIN_VERSION_ghc_lib_parser)
-#  if  !MIN_VERSION_ghc_lib_parser(1,  0,  0) || MIN_VERSION_ghc_lib_parser(8, 10, 0)
+#  if !MIN_VERSION_ghc_lib_parser(1,  0,  0) || MIN_VERSION_ghc_lib_parser(8, 10, 0)
   , testCase "ImportQualifiedPost" $ do
       case parseImport "import Foo qualified" (flags `xopt_set` ImportQualifiedPost) of
         POk _ (L _ decl) -> assertBool "expected postpositive" (isImportQualifiedPost . ideclQualified $ decl)
@@ -352,9 +463,12 @@ dynFlagsTests = testGroup "DynFlags tests"
 #endif
   ]
   where
-    flags = unsafeGlobalDynFlags
+    flags = defaultDynFlags fakeSettings fakeLlvmConfig
+#if defined (GHCLIB_API_HEAD) || defined (GHCLIB_API_902)
+    report flags msgs = concat [ showSDoc flags msg | msg <- pprMsgEnvelopeBagWithLoc msgs ]
+#else
     report flags msgs = concat [ showSDoc flags msg | msg <- pprErrMsgBagWithLoc msgs ]
-
+#endif
 nameTests :: TestTree
 nameTests = testGroup "Name tests"
   [ testCase "modName (1)" $
